TOPICS = {
    "dl19": "dl19-passage",
    "dl20": "dl20-passage",
    "dl21": "dl21-passage",
    "dl22": "dl22-passage",
    "covid": "beir-v1.0.0-trec-covid-test",
    "arguana": "beir-v1.0.0-arguana-test",
    "touche": "beir-v1.0.0-webis-touche2020-test",
    "news": "beir-v1.0.0-trec-news-test",
    "scifact": "beir-v1.0.0-scifact-test",
    "fiqa": "beir-v1.0.0-fiqa-test",
    "scidocs": "beir-v1.0.0-scidocs-test",
    "nfc": "beir-v1.0.0-nfcorpus-test",
    "quora": "beir-v1.0.0-quora-test",
    "dbpedia": "beir-v1.0.0-dbpedia-entity-test",
    "fever": "beir-v1.0.0-fever-test",
    "robust04": "beir-v1.0.0-robust04-test",
    "signal": "beir-v1.0.0-signal1m-test",
    "mrtydi-ar": "mrtydi-v1.1-arabic-test",
    "mrtydi-bn": "mrtydi-v1.1-bengali-test",
    "mrtydi-fi": "mrtydi-v1.1-finnish-test",
    "mrtydi-id": "mrtydi-v1.1-indonesian-test",
    "mrtydi-ja": "mrtydi-v1.1-japanese-test",
    "mrtydi-ko": "mrtydi-v1.1-korean-test",
    "mrtydi-ru": "mrtydi-v1.1-russian-test",
    "mrtydi-sw": "mrtydi-v1.1-swahili-test",
    "mrtydi-te": "mrtydi-v1.1-telugu-test",
    "mrtydi-th": "mrtydi-v1.1-thai-test",
}
